@@ -30,7 +30,6 @@ void loglikelihood(double *pars, int *tdcov, double *objf, int *grad, int *hess,
 		mp1[ng].reset(totalstates,nrcomp);
 		mpt[ng].reset(totalstates,nrcomp);
 		mptfinal[ng].reset(nrcomp);
-		
 	}
 	
 	hess[0]=0; // hessian not supported
@@ -67,10 +66,12 @@ void loglikelihood(double *pars, int *tdcov, double *objf, int *grad, int *hess,
 		models.setPars(pars);
 	}
 	
-	double loglike = 0.0;
-		
+	double loglike = 0.0;	
+	matrix indLogl;
+	
 	//lystig and hughes likelihood: scaling is done with factors of t-1!!
 	for(int ng=0; ng<ngroups; ng++) { 
+		indLogl.reset(ngdat.getIndReal(ng),1);
 		for(int nr=1; nr<=ngdat.getIndReal(ng); nr++) { 
 			double sf1=0.0;
 			double sft=0.0;
@@ -86,7 +87,7 @@ void loglikelihood(double *pars, int *tdcov, double *objf, int *grad, int *hess,
 			} //component fwd1's computed
 			//get their sum
 			for(int cp=0; cp < nrcomp; cp++) sf1 += models.fwd1[cp].msum();
-			loglike += log(sf1);
+			indLogl(nr) += log(sf1);
 			//compute gradient vectors for t=1
 			if(grad[0]) {
 				int prst=0;
@@ -112,7 +113,7 @@ void loglikelihood(double *pars, int *tdcov, double *objf, int *grad, int *hess,
 //						for(int np2=1; np2<=npars[cp]; np2++) {
 //							for(int st=1; st<=nstates[cp]; st++) {
 //								models.omega1[cp][st-1](np1,np2) = models.mixprop(ng+1,cp+1)*
-///									(models.mods[ng*nrcomp+cp].derin(np2,st)*models.mods[ng*nrcomp+cp].derobs(np1,st,ngdat(ng+1,nr,t)) +
+//									(models.mods[ng*nrcomp+cp].derin(np2,st)*models.mods[ng*nrcomp+cp].derobs(np1,st,ngdat(ng+1,nr,t)) +
 //									 models.mods[ng*nrcomp+cp].derin(np1,st)*models.mods[ng*nrcomp+cp].derobs(np2,st,ngdat(ng+1,nr,t)) +
 //									 models.mods[ng*nrcomp+cp].getInProb(st)*models.mods[ng*nrcomp+cp].hesobs(np1,np2,st,ngdat(ng+1,nr,t)));
 //							}
@@ -149,7 +150,7 @@ void loglikelihood(double *pars, int *tdcov, double *objf, int *grad, int *hess,
 						models.fwdt[cp] = (had(transpose(models.mods[ng*nrcomp+cp].getTrProb())*models.fwd1[cp],models.mods[ng*nrcomp+cp].getDataProb(ngdat(ng+1,nr,t))))*(1.0/sf1);
 						sft += models.fwdt[cp].msum();
 					} 
-					loglike += log(sft);
+					indLogl(nr) += log(sft);
 					// zero the workspace vars psit and mpt
 					if(grad[0]) {
 						int prst=0;
@@ -203,6 +204,7 @@ void loglikelihood(double *pars, int *tdcov, double *objf, int *grad, int *hess,
 					sf1=sft;
 				}
 			}// end of current realization at t=length
+			loglike += ngdat.getWeight(ng,nr)*indLogl(nr);
 			// build up the sums of psit over realizations (sum over states psitfinal+=psiT/fwdT) for the current group
 			if(grad[0]) {
 				for(int cp1=0; cp1<nrcomp; cp1++) {
@@ -211,8 +213,8 @@ void loglikelihood(double *pars, int *tdcov, double *objf, int *grad, int *hess,
 						for(int st=1; st<=nstates[cp1]; st++) {
 							npsum += models.psi1[ng*nrcomp+cp1](st,np);
 						}
-						models.psitfinal[ng*nrcomp+cp1](np) += npsum/sf1;
-						if(grInd[0]) gradIndi(nr,np+1) = npsum/sf1;
+						models.psitfinal[ng*nrcomp+cp1](np) += ngdat.getWeight(ng,nr)*npsum/sf1;
+						if(grInd[0]) gradIndi(nr,np+1) = ngdat.getWeight(ng,nr)*npsum/sf1;
 					}
 					int prst=0;
 					double mpsum=0.0;
@@ -223,11 +225,12 @@ void loglikelihood(double *pars, int *tdcov, double *objf, int *grad, int *hess,
 							mpsum += mp1[ng](inst,cp1+1);
 						}
 					}
-					mptfinal[ng](cp1+1) += mpsum/sf1;
-					if(grInd[0]) gradIndi(nr,1) = mpsum/sf1;
+					mptfinal[ng](cp1+1) += ngdat.getWeight(ng,nr)*mpsum/sf1;
+					if(grInd[0]) gradIndi(nr,1) = ngdat.getWeight(ng,nr)*mpsum/sf1;
 				}
 			}
 			// build up omegafinal terms, ie sum over states, realizations and groups
+			// case weighting not applied here yet
 			if(hess[0]) {
 				for(int cp=0; cp<nrcomp; cp++) {
 					for(int np1=1; np1<=npars[cp]; np1++) {
@@ -244,7 +247,7 @@ void loglikelihood(double *pars, int *tdcov, double *objf, int *grad, int *hess,
 			if(printlevel>29) Rprintf("logLikelihood: %f after case nr %d \n", loglike, nr);
 		} // end realizations
 	} // end groups, likelihood computed
-	
+		
 	//set the loglikelihood
 	objf[0] = scale[0]*loglike; 
 		
@@ -565,13 +568,14 @@ void multiDataSetUp(int *ng, int *print) {
 	if(printlevel>19) ngdat.summary();
 }
 
-void ngDataSetUp(int *groupnr, double *data, int *vars, int *modes, int *indReal, int *lengths, int *xmiss, int *print) {
+void ngDataSetUp(int *groupnr, double *data, int *vars, int *modes, int *indReal, int *lengths, double *weights, int *xmiss, int *print) {
 	printlevel=print[0];
 	int gnr=groupnr[0];
 	int xm=xmiss[0];
 	if(gnr<1||gnr>ngdat.getNGroups()) error("Wrong groupnr in data set up");
 	if(printlevel>19) Rprintf("Setting data for group %d \n", gnr);
-	ngdat.data[gnr-1].mtsdata(data,vars[0],modes,indReal[0],lengths,xm);
+	if(printlevel>19) ngdat.data[gnr-1].summary();
+	ngdat.data[gnr-1].mtsdata(data,vars[0],modes,indReal[0],lengths,weights,xm);
 	if(printlevel>19) ngdat.data[gnr-1].summary();
 	if(printlevel>29) ngdat.data[gnr-1].print();
 }
@@ -582,13 +586,13 @@ void multiCovSetUp(int *ng, int *print) {
 	if(printlevel>19) ngcov.summary();
 }
 
-void ngCovSetUp(int *groupnr, double *data, int *vars, int *modes, int *indReal, int *lengths, int *xmiss, int *print) {
+void ngCovSetUp(int *groupnr, double *data, int *vars, int *modes, int *indReal, int *lengths, double *weights, int *xmiss, int *print) {
 	printlevel=print[0];
 	int gnr=groupnr[0];
 	int xm=xmiss[0];
 	if(gnr<1||gnr>ngcov.getNGroups()) error("Wrong groupnr in covariates set up");
 	if(printlevel>19) Rprintf("Setting data for group %d \n", gnr);
-	ngcov.data[gnr-1].mtsdata(data,vars[0],modes,indReal[0],lengths,xm);
+	ngcov.data[gnr-1].mtsdata(data,vars[0],modes,indReal[0],lengths,weights,xm);
 	if(printlevel>19) ngcov.data[gnr-1].summary();
 }
 
@@ -599,9 +603,7 @@ void ngCovSetUp(int *groupnr, double *data, int *vars, int *modes, int *indReal,
 /*															*/
 /************************************************************/
 
-
 /*
-
 // MAIN OPT ROUTINE CALLED FROM R
 // This can be used for any problem ... provided that the loglikelihood function is provided
 void npsolc(int *n, int *nclin, int *ncnln, int *ldA, int *ldJu, int *ldR, 
@@ -758,13 +760,10 @@ void funobj(int *mode, int *noptpars, double *optpars, double *objf, double *g,
 
 //derivatives of non-linear contraints of funobj (dummy function because there are no non-linear constraints)
 void funcon(int *mode, int *ncnln, int *n, int *ldJ, int *needc, double *x, double *c, double *cJac, int *nstate) { }
-
+*/
 
 // END NPSOL FUNCTIONS
 
-*/
-
 // UNCOMMNENT ABOVE TO USE NPSOL FUNCTIONS
 
-} // end extern "C"
-
+} // end extern "C"
\ No newline at end of file
